@@ -69,12 +69,32 @@ static const char KNOWN_FILES_KEY[] =
 static const char DISABLE_INDICATOR_KEY[] =
     "/desktop/gnome/peripherals/keyboard/general/disable_indicator";
 
+static const char DUPLICATE_LEDS_KEY[] =
+    "/desktop/gnome/peripherals/keyboard/general/duplicate_leds";
+
 static const char *gdm_keyboard_layout = NULL;
 
 static GtkStatusIcon *icon = NULL;
 
 static GHashTable *preview_dialogs = NULL;
 
+static Atom caps_lock;
+static Atom num_lock;
+static Atom scroll_lock;
+
+static GtkStatusIcon *indicator_icons[3];
+static const gchar *indicator_on_icon_names[] = {
+	"scrolllock_on",
+	"numlock_on",
+	"capslock_on"
+};
+
+static const gchar *indicator_off_icon_names[] = {
+	"scrolllock_off",
+	"numlock_off",
+	"capslock_off"
+};
+
 #define noGSDKX
 
 #ifdef GSDKX
@@ -116,13 +136,13 @@ activation_error (void)
 						     _
 						     ("Error activating XKB configuration.\n"
 						      "It can happen under various circumstances:\n"
-						      "- a bug in libxklavier library\n"
-						      "- a bug in X server (xkbcomp, xmodmap utilities)\n"
-						      "- X server with incompatible libxkbfile implementation\n\n"
+						      " • a bug in libxklavier library\n"
+						      " • a bug in X server (xkbcomp, xmodmap utilities)\n"
+						      " • X server with incompatible libxkbfile implementation\n\n"
 						      "X server version data:\n%s\n%d\n%s\n"
 						      "If you report this situation as a bug, please include:\n"
-						      "- The result of <b>%s</b>\n"
-						      "- The result of <b>%s</b>"),
+						      " • The result of <b>%s</b>\n"
+						      " • The result of <b>%s</b>"),
 						     vendor,
 						     release,
 						     badXFree430Release
@@ -130,7 +150,7 @@ activation_error (void)
 						     _
 						     ("You are using XFree 4.3.0.\n"
 						      "There are known problems with complex XKB configurations.\n"
-						      "Try using a simpler configuration or taking a fresher version of XFree software.")
+						      "Try using a simpler configuration or using a later version of the XFree software.")
 						     : "",
 						     "xprop -root | grep XKB",
 						     "gconftool-2 -R /desktop/gnome/peripherals/keyboard/kbd");
@@ -142,6 +162,9 @@ activation_error (void)
 static void
 apply_desktop_settings (void)
 {
+	GConfClient *conf_client;
+	gboolean show_leds;
+	int i;
 	if (!inited_ok)
 		return;
 
@@ -150,6 +173,16 @@ apply_desktop_settings (void)
 	/* again, probably it would be nice to compare things
 	   before activating them */
 	gkbd_desktop_config_activate (&current_config);
+
+	conf_client = gconf_client_get_default ();
+	show_leds =
+	    gconf_client_get_bool (conf_client, DUPLICATE_LEDS_KEY, NULL);
+	g_object_unref (conf_client);
+	for (i = sizeof (indicator_icons) / sizeof (indicator_icons[0]);
+	     --i >= 0;) {
+		gtk_status_icon_set_visible (indicator_icons[i],
+					     show_leds);
+	}
 }
 
 static void
@@ -392,7 +425,7 @@ status_icon_popup_menu_cb (GtkStatusIcon * icon, guint button, guint time)
 	int i = 0;
 	gchar **current_name = gkbd_status_get_group_names ();
 
-	GtkWidget *item = gtk_menu_item_new_with_mnemonic (_("_Groups"));
+	GtkWidget *item = gtk_menu_item_new_with_mnemonic (_("_Layouts"));
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item),
@@ -405,7 +438,7 @@ status_icon_popup_menu_cb (GtkStatusIcon * icon, guint button, guint time)
 			  NULL);
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
 
-	item = gtk_menu_item_new_with_mnemonic (_("Show Current _Layout"));
+	item = gtk_menu_item_new_with_mnemonic (_("Show _Current Layout"));
 	gtk_widget_show (item);
 	g_signal_connect (item, "activate", popup_menu_show_layout, NULL);
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
@@ -848,11 +881,74 @@ gsd_keyboard_new_device (XklEngine * engine)
 	apply_xkb_settings ();
 }
 
+static void
+gsd_keyboard_update_indicator_icons ()
+{
+	Bool state;
+	int new_state, i;
+	Display *display = GDK_DISPLAY ();
+	XkbGetNamedIndicator (display, caps_lock, NULL, &state,
+			      NULL, NULL);
+	new_state = state ? 1 : 0;
+	XkbGetNamedIndicator (display, num_lock, NULL, &state, NULL, NULL);
+	new_state <<= 1;
+	new_state |= (state ? 1 : 0);
+	XkbGetNamedIndicator (display, scroll_lock, NULL, &state,
+			      NULL, NULL);
+	new_state <<= 1;
+	new_state |= (state ? 1 : 0);
+	xkl_debug (160, "Indicators state: %d\n", new_state);
+
+
+	for (i = sizeof (indicator_icons) / sizeof (indicator_icons[0]);
+	     --i >= 0;) {
+		gtk_status_icon_set_from_icon_name (indicator_icons[i],
+						    (new_state & (1 << i))
+						    ?
+						    indicator_on_icon_names
+						    [i] :
+						    indicator_off_icon_names
+						    [i]);
+	}
+}
+
+static void
+gsd_keyboard_state_changed (XklEngine * engine, XklEngineStateChange type,
+			    gint new_group, gboolean restore)
+{
+	xkl_debug (160,
+		   "State changed: type %d, new group: %d, restore: %d.\n",
+		   type, new_group, restore);
+	if (type == INDICATORS_CHANGED) {
+		gsd_keyboard_update_indicator_icons ();
+	}
+}
+
 void
 gsd_keyboard_xkb_init (GConfClient * client,
 		       GsdKeyboardManager * kbd_manager)
 {
+	int i;
+	Display *display = GDK_DISPLAY ();
 	gnome_settings_profile_start (NULL);
+
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
+					   DATADIR G_DIR_SEPARATOR_S
+					   "icons");
+
+	caps_lock = XInternAtom (display, "Caps Lock", False);
+	num_lock = XInternAtom (display, "Num Lock", False);
+	scroll_lock = XInternAtom (display, "Scroll Lock", False);
+
+	for (i = sizeof (indicator_icons) / sizeof (indicator_icons[0]);
+	     --i >= 0;) {
+		indicator_icons[i] =
+		    gtk_status_icon_new_from_icon_name
+		    (indicator_off_icon_names[i]);
+	}
+
+	gsd_keyboard_update_indicator_icons ();
+
 #ifdef GSDKX
 	xkl_set_debug_level (200);
 	logfile = fopen ("/tmp/gsdkx.log", "a");
@@ -860,7 +956,7 @@ gsd_keyboard_xkb_init (GConfClient * client,
 #endif
 	manager = kbd_manager;
 	gnome_settings_profile_start ("xkl_engine_get_instance");
-	xkl_engine = xkl_engine_get_instance (GDK_DISPLAY ());
+	xkl_engine = xkl_engine_get_instance (display);
 	gnome_settings_profile_end ("xkl_engine_get_instance");
 	if (xkl_engine) {
 		inited_ok = TRUE;
@@ -899,6 +995,9 @@ gsd_keyboard_xkb_init (GConfClient * client,
 			g_signal_connect (xkl_engine, "X-new-device",
 					  G_CALLBACK
 					  (gsd_keyboard_new_device), NULL);
+		g_signal_connect (xkl_engine, "X-state-changed",
+				  G_CALLBACK
+				  (gsd_keyboard_state_changed), NULL);
 
 		gnome_settings_profile_start ("xkl_engine_start_listen");
 		xkl_engine_start_listen (xkl_engine,
@@ -914,6 +1013,7 @@ gsd_keyboard_xkb_init (GConfClient * client,
 		gnome_settings_profile_end ("apply_xkb_settings");
 	}
 	preview_dialogs = g_hash_table_new (g_direct_hash, g_direct_equal);
+
 	gnome_settings_profile_end (NULL);
 }
 
@@ -921,11 +1021,18 @@ void
 gsd_keyboard_xkb_shutdown (void)
 {
 	GConfClient *client;
+	int i;
 
 	pa_callback = NULL;
 	pa_callback_user_data = NULL;
 	manager = NULL;
 
+	for (i = sizeof (indicator_icons) / sizeof (indicator_icons[0]);
+	     --i >= 0;) {
+		g_object_unref (G_OBJECT (indicator_icons[i]));
+		indicator_icons[i] = NULL;
+	}
+
 	g_hash_table_destroy (preview_dialogs);
 
 	if (!inited_ok)
